import 'dart:async';
import 'dart:math';

import 'package:audioplayers/audioplayers.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:twister/cubit/game_cubit.dart';
import 'package:twister/models/move.dart';
import 'package:twister/models/twister_color.dart';
import 'package:twister/models/twister_member.dart';
import 'package:twister/ui/widgets/show_move.dart';

class Game extends StatefulWidget {
  const Game({super.key});

  @override
  State<Game> createState() => _GameState();
}

class _GameState extends State<Game> {
  final player = AudioPlayer();
  bool shuffling = false;
  Move? shuffledMove;

  void animate() {
    const interval = Duration(milliseconds: 200);
    int iterationsLeft = 10;

    shuffling = true;
    shuffledMove = null;

    setState(() {});
    Timer.periodic(
      interval,
      (Timer timer) {
        if (iterationsLeft > 1) {
          shuffledMove = Move.pickRandom();
        }

        if (iterationsLeft == 1) {
          shuffledMove = null;
        }

        if (iterationsLeft == 0) {
          shuffledMove = null;
          pickNewMove();

          timer.cancel();
          shuffling = false;
        }

        setState(() {});
        iterationsLeft--;
      },
    );
  }

  void pickNewMove() {
    Move newMove = Move.pickRandom();

    BlocProvider.of<GameCubit>(context).setValues(
      move: newMove,
    );

    player.play(AssetSource(newMove.toSoundAsset()));
  }

  Widget buildCurrentStateWidget(List<Move>? history) {
    Map<String, TwisterColor?> currentState = {};

    history?.forEach((move) {
      currentState[move.member.toString()] = move.color;
    });

    TwisterMember leftHand = TwisterMember(value: TwisterAllowedMembers.leftHand);
    TwisterMember rightHand = TwisterMember(value: TwisterAllowedMembers.rightHand);
    TwisterMember leftFoot = TwisterMember(value: TwisterAllowedMembers.leftFoot);
    TwisterMember rightFoot = TwisterMember(value: TwisterAllowedMembers.rightFoot);

    Move leftHandMove = Move.createFrom(
      member: leftHand,
      color: currentState[leftHand.toString()],
    );
    Move rightHandMove = Move.createFrom(
      member: rightHand,
      color: currentState[rightHand.toString()],
    );
    Move leftFootMove = Move.createFrom(
      member: leftFoot,
      color: currentState[leftFoot.toString()],
    );
    Move rightFootMove = Move.createFrom(
      member: rightFoot,
      color: currentState[rightFoot.toString()],
    );

    return Padding(
      padding: const EdgeInsets.all(30),
      child: Table(
        children: [
          TableRow(
            children: [
              Padding(
                padding: const EdgeInsets.all(10),
                child: ShowMove(move: leftHandMove),
              ),
              Padding(
                padding: const EdgeInsets.all(10),
                child: ShowMove(move: rightHandMove),
              ),
            ],
          ),
          TableRow(
            children: [
              Padding(
                padding: const EdgeInsets.all(10),
                child: ShowMove(move: leftFootMove),
              ),
              Padding(
                padding: const EdgeInsets.all(10),
                child: ShowMove(move: rightFootMove),
              ),
            ],
          )
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<GameCubit, GameState>(
      builder: (BuildContext context, GameState gameState) {
        return Column(
          children: [
            GestureDetector(
              child: shuffling
                  ? Transform.rotate(
                      angle: 2 * pi * Random().nextDouble(),
                      child: ShowMove(move: shuffledMove ?? Move.createNull()),
                    )
                  : ShowMove(move: gameState.move ?? Move.createNull()),
              onTap: () {
                animate();
              },
            ),
            GestureDetector(
              child: buildCurrentStateWidget(gameState.history),
              onTap: () {
                BlocProvider.of<GameCubit>(context).deleteHistory();
              },
            ),
          ],
        );
      },
    );
  }
}
